package de.uks.pmws1112.hexentanz.model;

import java.util.*;

public class HexentanzGame
{
   /**
    * <pre>
    *           0..1     currentPlayer     0..1
    * Game ------------------------- Player
    *           currentPlayerRev               currentPlayer
    * </pre>
    */
   public static final String PROPERTY_CURRENT_PLAYER = "currentPlayer";

   private Player currentPlayer;

   public boolean setCurrentPlayer (Player value)
   {
      boolean changed = false;

      if (this.currentPlayer != value)
      {
      
         Player oldValue = this.currentPlayer;
         HexentanzGame source = this;
         if (this.currentPlayer != null)
         {
            this.currentPlayer = null;
            oldValue.setCurrentPlayerRev (null);
         }
         this.currentPlayer = value;

         if (value != null)
         {
            value.setCurrentPlayerRev (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public HexentanzGame withCurrentPlayer (Player value)
   {
      setCurrentPlayer (value);
      return this;
   }

   public Player getCurrentPlayer ()
   {
      return this.currentPlayer;
   }

   /**
    * <pre>
    *           0..1     dice     0..1
    * Game ------------------------- Dice
    *           game               dice
    * </pre>
    */
   public static final String PROPERTY_DICE = "dice";

   private Dice dice;

   public boolean setDice (Dice value)
   {
      boolean changed = false;

      if (this.dice != value)
      {
      
         Dice oldValue = this.dice;
         HexentanzGame source = this;
         if (this.dice != null)
         {
            this.dice = null;
            oldValue.setGame (null);
         }
         this.dice = value;

         if (value != null)
         {
            value.setGame (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public HexentanzGame withDice (Dice value)
   {
      setDice (value);
      return this;
   }

   public Dice getDice ()
   {
      return this.dice;
   }

   /**
    * <pre>
    *           0..1     fields     0..*
    * Game ------------------------- Field
    *           game               fields
    * </pre>
    */
   public static final String PROPERTY_FIELDS = "fields";

   private HashSet<Field> fields;

   public Set<? extends Field> getFields()
   {
      return ((this.fields == null)
              ? Collections.EMPTY_SET
              : Collections.unmodifiableSet(this.fields));
   }

   public boolean addToFields (Field value)
   {
      boolean changed = false;

      if (value != null)
      {
         if (this.fields == null)
         {
            this.fields = new HashSet<Field> ();
         }
      
         changed = this.fields.add (value);
         if (changed)
         {
            value.setGame (this);
         }
      
      }
      return changed;
   }

   public HexentanzGame withFields (Field value)
   {
      addToFields (value);
      return this;
   }

   public HexentanzGame withoutFields (Field value)
   {
      removeFromFields (value);
      return this;
   }


   public boolean removeFromFields (Field value)
   {
      boolean changed = false;

      if ((this.fields != null) && (value != null))
      {
      
         changed = this.fields.remove (value);
         if (changed)
         {
            value.setGame (null);
         }
      
      }
      return changed;
   }

   public void removeAllFromFields (){
   
      Field tmpValue;
      Iterator<? extends Field> iter = this.iteratorOfFields ();
      while (iter.hasNext ())
      {
         tmpValue = (Field) iter.next ();
         this.removeFromFields (tmpValue);
      }
   
   }

   public boolean hasInFields (Field value)
   {
      return ((this.fields != null) &&
              (value != null) &&
              this.fields.contains (value));
   }

   public Iterator<? extends Field> iteratorOfFields ()
   {
      return ((this.fields == null)
              ? new Iterator<Field>()
            	{
					@Override
					public boolean hasNext()
					{
						return false;
					}
	
					@Override
					public Field next()
					{
						return null;
					}
	
					@Override
					public void remove() {}
				}
              : this.fields.iterator ());
   }

   public int sizeOfFields ()
   {
      return ((this.fields == null)
              ? 0
              : this.fields.size ());
   }

   /**
    * <pre>
    *           0..1     players     0..*
    * Game ------------------------- Player
    *           game               players
    * </pre>
    */
   public static final String PROPERTY_PLAYERS = "players";

   private HashSet<Player> players;

   public Set<? extends Player> getPlayers()
   {
      return ((this.players == null)
              ? Collections.EMPTY_SET
              : Collections.unmodifiableSet(this.players));
   }

   public boolean addToPlayers (Player value)
   {
      boolean changed = false;

      if (value != null)
      {
         if (this.players == null)
         {
            this.players = new HashSet<Player> ();
         }
      
         changed = this.players.add (value);
         if (changed)
         {
            value.setGame (this);
         }
      
      }
      return changed;
   }

   public HexentanzGame withPlayers (Player value)
   {
      addToPlayers (value);
      return this;
   }

   public HexentanzGame withoutPlayers (Player value)
   {
      removeFromPlayers (value);
      return this;
   }


   public boolean removeFromPlayers (Player value)
   {
      boolean changed = false;

      if ((this.players != null) && (value != null))
      {
      
         changed = this.players.remove (value);
         if (changed)
         {
            value.setGame (null);
         }
      
      }
      return changed;
   }

   public void removeAllFromPlayers (){
   
      Player tmpValue;
      Iterator<? extends Player> iter = this.iteratorOfPlayers ();
      while (iter.hasNext ())
      {
         tmpValue = (Player) iter.next ();
         this.removeFromPlayers (tmpValue);
      }
   
   }

   public boolean hasInPlayers (Player value)
   {
      return ((this.players != null) &&
              (value != null) &&
              this.players.contains (value));
   }

   public Iterator<? extends Player> iteratorOfPlayers ()
   {
      return ((this.players == null)
              ? new Iterator<Player>()
            	{
					@Override
					public boolean hasNext()
					{
						return false;
					}
	
					@Override
					public Player next()
					{
						return null;
					}
	
					@Override
					public void remove() {}
      			}
              : this.players.iterator ());
   }

   public int sizeOfPlayers ()
   {
      return ((this.players == null)
              ? 0
              : this.players.size ());
   }

   public void startGame ()
   {
   }

   /**
    * <pre>
    *           0..1     winner     0..1
    * Game ------------------------- Player
    *           game2               winner
    * </pre>
    */
   public static final String PROPERTY_WINNER = "winner";

   private Player winner;

   public boolean setWinner (Player value)
   {
      boolean changed = false;

      if (this.winner != value)
      {
      
         Player oldValue = this.winner;
         HexentanzGame source = this;
         if (this.winner != null)
         {
            this.winner = null;
            oldValue.setGame2 (null);
         }
         this.winner = value;

         if (value != null)
         {
            value.setGame2 (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public HexentanzGame withWinner (Player value)
   {
      setWinner (value);
      return this;
   }

   public Player getWinner ()
   {
      return this.winner;
   }

   /**
    * <pre>
    *           0..1     witches     0..*
    * Game ------------------------- Witch
    *           game               witches
    * </pre>
    */
   public static final String PROPERTY_WITCHES = "witches";

   private HashSet<Witch> witches;

   public Set<? extends Witch> getWitches()
   {
      return ((this.witches == null)
              ? Collections.EMPTY_SET
              : Collections.unmodifiableSet(this.witches));
   }

   public boolean addToWitches (Witch value)
   {
      boolean changed = false;

      if (value != null)
      {
         if (this.witches == null)
         {
            this.witches = new HashSet<Witch> ();
         }
      
         changed = this.witches.add (value);
         if (changed)
         {
            value.setGame (this);
         }
      
      }
      return changed;
   }

   public HexentanzGame withWitches (Witch value)
   {
      addToWitches (value);
      return this;
   }

   public HexentanzGame withoutWitches (Witch value)
   {
      removeFromWitches (value);
      return this;
   }


   public boolean removeFromWitches (Witch value)
   {
      boolean changed = false;

      if ((this.witches != null) && (value != null))
      {
      
         changed = this.witches.remove (value);
         if (changed)
         {
            value.setGame (null);
         }
      
      }
      return changed;
   }

   public void removeAllFromWitches (){
   
      Witch tmpValue;
      Iterator<? extends Witch> iter = this.iteratorOfWitches ();
      while (iter.hasNext ())
      {
         tmpValue = (Witch) iter.next ();
         this.removeFromWitches (tmpValue);
      }
   
   }

   public boolean hasInWitches (Witch value)
   {
      return ((this.witches != null) &&
              (value != null) &&
              this.witches.contains (value));
   }

   public Iterator<? extends Witch> iteratorOfWitches ()
   {
      return ((this.witches == null)
              ? new Iterator<Witch>()
            	{
					@Override
					public boolean hasNext()
					{
						return false;
					}
	
					@Override
					public Witch next()
					{
						return null;
					}
	
					@Override
					public void remove() {}
				}
              : this.witches.iterator ());
   }

   public int sizeOfWitches ()
   {
      return ((this.witches == null)
              ? 0
              : this.witches.size ());
   }

   public void removeYou()
   {
      this.setCurrentPlayer (null);
      this.setDice (null);
      this.removeAllFromFields ();
      this.removeAllFromPlayers ();
      this.setWinner (null);
      this.removeAllFromWitches ();
   }
   
   public void init(String[] playerNames )
   {
	   //TODO !!Hausaufgabe 4!!
   }
}


