package de.uks.pmws1112.hexentanz.model;

public class Player
{
   /**
    * <pre>
    *           0..1     currentPlayer     0..1
    * Player ------------------------- Game
    *           currentPlayer               currentPlayerRev
    * </pre>
    */
   public static final String PROPERTY_CURRENT_PLAYER_REV = "currentPlayerRev";

   private HexentanzGame currentPlayerRev;

   public boolean setCurrentPlayerRev (HexentanzGame value)
   {
      boolean changed = false;

      if (this.currentPlayerRev != value)
      {
      
         HexentanzGame oldValue = this.currentPlayerRev;
         Player source = this;
         if (this.currentPlayerRev != null)
         {
            this.currentPlayerRev = null;
            oldValue.setCurrentPlayer (null);
         }
         this.currentPlayerRev = value;

         if (value != null)
         {
            value.setCurrentPlayer (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withCurrentPlayerRev (HexentanzGame value)
   {
      setCurrentPlayerRev (value);
      return this;
   }

   public HexentanzGame getCurrentPlayerRev ()
   {
      return this.currentPlayerRev;
   }

   /**
    * <pre>
    *           0..1     dice     0..1
    * Player ------------------------- Dice
    *           player               dice
    * </pre>
    */
   public static final String PROPERTY_DICE = "dice";

   private Dice dice;

   public boolean setDice (Dice value)
   {
      boolean changed = false;

      if (this.dice != value)
      {
      
         Dice oldValue = this.dice;
         Player source = this;
         if (this.dice != null)
         {
            this.dice = null;
            oldValue.setPlayer (null);
         }
         this.dice = value;

         if (value != null)
         {
            value.setPlayer (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withDice (Dice value)
   {
      setDice (value);
      return this;
   }

   public Dice getDice ()
   {
      return this.dice;
   }

   /**
    * <pre>
    *           0..1     winner     0..1
    * Player ------------------------- Game
    *           winner               game2
    * </pre>
    */
   public static final String PROPERTY_GAME2 = "game2";

   private HexentanzGame game2;

   public boolean setGame2 (HexentanzGame value)
   {
      boolean changed = false;

      if (this.game2 != value)
      {
      
         HexentanzGame oldValue = this.game2;
         Player source = this;
         if (this.game2 != null)
         {
            this.game2 = null;
            oldValue.setWinner (null);
         }
         this.game2 = value;

         if (value != null)
         {
            value.setWinner (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withGame2 (HexentanzGame value)
   {
      setGame2 (value);
      return this;
   }

   public HexentanzGame getGame2 ()
   {
      return this.game2;
   }

   /**
    * <pre>
    *           0..*     players     0..1
    * Player ------------------------- Game
    *           players               game
    * </pre>
    */
   public static final String PROPERTY_GAME = "game";

   private HexentanzGame game;

   public boolean setGame (HexentanzGame value)
   {
      boolean changed = false;

      if (this.game != value)
      {
      
         HexentanzGame oldValue = this.game;
         Player source = this;
         if (this.game != null)
         {
            this.game = null;
            oldValue.removeFromPlayers (this);
         }
         this.game = value;

         if (value != null)
         {
            value.addToPlayers (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withGame (HexentanzGame value)
   {
      setGame (value);
      return this;
   }

   public HexentanzGame getGame ()
   {
      return this.game;
   }

   public static final String PROPERTY_INSTRUCTION = "instruction";

   private String instruction;

   public void setInstruction (String value)
   {
      if ( ! this.instruction.equals (value))
      {
         String oldValue = this.instruction;
         this.instruction = value;
      }
   }

   public Player withInstruction (String value)
   {
      setInstruction (value);
      return this;
   }

   public String getInstruction ()
   {
      return this.instruction;
   }

   public static final String PROPERTY_NAME = "name";

   private String name;

   public void setName (String value)
   {
      if (this.name == null || ! this.name.equals (value))
      {
         String oldValue = this.name;
         this.name = value;
      }
   }

   public Player withName (String value)
   {
      setName (value);
      return this;
   }

   public String getName ()
   {
      return this.name;
   }

   /**
    * <pre>
    *           0..1     next     0..1
    * Player ------------------------- Player
    *           previous               next
    * </pre>
    */
   public static final String PROPERTY_NEXT = "next";

   private Player next;

   public boolean setNext (Player value)
   {
      boolean changed = false;

      if (this.next != value)
      {
      
         Player oldValue = this.next;
         Player source = this;
         if (this.next != null)
         {
            this.next = null;
            oldValue.setPrevious (null);
         }
         this.next = value;

         if (value != null)
         {
            value.setPrevious (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withNext (Player value)
   {
      setNext (value);
      return this;
   }

   public Player getNext ()
   {
      return this.next;
   }

   /**
    * <pre>
    *           0..1     next     0..1
    * Player ------------------------- Player
    *           next               previous
    * </pre>
    */
   public static final String PROPERTY_PREVIOUS = "previous";

   private Player previous;

   public boolean setPrevious (Player value)
   {
      boolean changed = false;

      if (this.previous != value)
      {
      
         Player oldValue = this.previous;
         Player source = this;
         if (this.previous != null)
         {
            this.previous = null;
            oldValue.setNext (null);
         }
         this.previous = value;

         if (value != null)
         {
            value.setNext (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withPrevious (Player value)
   {
      setPrevious (value);
      return this;
   }

   public Player getPrevious ()
   {
      return this.previous;
   }

   /**
    * <pre>
    *           0..1     startField     0..1
    * Player ------------------------- StartField
    *           player               startField
    * </pre>
    */
   public static final String PROPERTY_START_FIELD = "startField";

   private StartField startField;

   public boolean setStartField (StartField value)
   {
      boolean changed = false;

      if (this.startField != value)
      {
      
         StartField oldValue = this.startField;
         Player source = this;
         if (this.startField != null)
         {
            this.startField = null;
            oldValue.setPlayer (null);
         }
         this.startField = value;

         if (value != null)
         {
            value.setPlayer (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public Player withStartField (StartField value)
   {
      setStartField (value);
      return this;
   }

   public StartField getStartField ()
   {
      return this.startField;
   }

   public void removeYou()
   {
      this.setCurrentPlayerRev (null);
      this.setDice (null);
      this.setGame2 (null);
      this.setGame (null);
      this.setNext (null);
      this.setPrevious (null);
      this.setStartField (null);
   }
}


