package de.uks.pmws1112.hexentanz.model;

import java.util.*;

public class StartField extends Field
{
   /**
    * <pre>
    *           0..1     startField     0..*
    * StartField ------------------------- HomeField
    *           startField               homeField
    * </pre>
    */
   public static final String PROPERTY_HOME_FIELD = "homeField";

   private HashSet<HomeField> homeField;

   public Set<? extends HomeField> getHomeField()
   {
      return ((this.homeField == null)
              ? Collections.EMPTY_SET
              : Collections.unmodifiableSet(this.homeField));
   }

   public boolean addToHomeField (HomeField value)
   {
      boolean changed = false;

      if (value != null)
      {
         if (this.homeField == null)
         {
            this.homeField = new HashSet<HomeField> ();

         }
      
         changed = this.homeField.add (value);
         if (changed)
         {
            value.setStartField (this);
         }
      
      }
      return changed;
   }

   public StartField withHomeField (HomeField value)
   {
      addToHomeField (value);
      return this;
   }

   public StartField withoutHomeField (HomeField value)
   {
      removeFromHomeField (value);
      return this;
   }


   public boolean removeFromHomeField (HomeField value)
   {
      boolean changed = false;

      if ((this.homeField != null) && (value != null))
      {
      
         changed = this.homeField.remove (value);
         if (changed)
         {
            value.setStartField (null);
         }
      
      }
      return changed;
   }

   public void removeAllFromHomeField ()
   {
      HomeField tmpValue;
      Iterator<? extends HomeField> iter = this.iteratorOfHomeField ();
      while (iter.hasNext ())
      {
         tmpValue = (HomeField) iter.next ();
         this.removeFromHomeField (tmpValue);
      }
   }

   public boolean hasInHomeField (HomeField value)
   {
      return ((this.homeField != null) &&
              (value != null) &&
              this.homeField.contains (value));
   }

   public Iterator<? extends HomeField> iteratorOfHomeField ()
   {
      return ((this.homeField == null)
              ? new Iterator<HomeField>()
            	{
					@Override
					public boolean hasNext()
					{
						return false;
					}
	
					@Override
					public HomeField next()
					{
						return null;
					}
	
					@Override
					public void remove() {}
				}
              : this.homeField.iterator ());
   }

   public int sizeOfHomeField ()
   {
      return ((this.homeField == null)
              ? 0
              : this.homeField.size ());
   }

   /**
    * <pre>
    *           0..1     startField     0..1
    * StartField ------------------------- Player
    *           startField               player
    * </pre>
    */
   public static final String PROPERTY_PLAYER = "player";

   private Player player;

   public boolean setPlayer (Player value)
   {
      boolean changed = false;

      if (this.player != value)
      {
      
         Player oldValue = this.player;
         StartField source = this;
         if (this.player != null)
         {
            this.player = null;
            oldValue.setStartField (null);
         }
         this.player = value;

         if (value != null)
         {
            value.setStartField (source);
         }
         changed = true;
      
      }
      return changed;
   }

   public StartField withPlayer (Player value)
   {
      setPlayer (value);
      return this;
   }

   public Player getPlayer ()
   {
      return this.player;
   }

   public void removeYou()
   {
      this.removeAllFromHomeField ();
      this.setPlayer (null);
      super.removeYou ();
   }
}


